# -*- coding: utf-8 -*-
# Namespaces, etc.

XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'

# PREMIS v. 2.2
PREMIS_2_2_VERSION = '2.2'
PREMIS_2_2_NAMESPACE = 'info:lc/xmlns/premis-v2'
PREMIS_2_2_XSD = 'http://www.loc.gov/standards/premis/v2/premis-v2-2.xsd'
PREMIS_2_2_SCHEMA_LOCATION = '{} {}'.format(
    PREMIS_2_2_NAMESPACE, PREMIS_2_2_XSD)
PREMIS_2_2_NAMESPACES = {
    'premis': PREMIS_2_2_NAMESPACE,
    'xsi': XSI_NAMESPACE
}
PREMIS_2_2_META = {
    'xsi:schema_location': PREMIS_2_2_SCHEMA_LOCATION,
    'version': PREMIS_2_2_VERSION
}

# PREMIS v. 3.0
PREMIS_3_0_VERSION = '3.0'
PREMIS_3_0_NAMESPACE = 'http://www.loc.gov/premis/v3'
PREMIS_3_0_XSD = 'http://www.loc.gov/standards/premis/v3/premis.xsd'
PREMIS_3_0_SCHEMA_LOCATION = '{} {}'.format(
    PREMIS_3_0_NAMESPACE, PREMIS_3_0_XSD)
PREMIS_3_0_NAMESPACES = {
    'premis': PREMIS_3_0_NAMESPACE,
    'xsi': XSI_NAMESPACE
}
PREMIS_3_0_META = {
    'xsi:schema_location': PREMIS_3_0_SCHEMA_LOCATION,
    'version': PREMIS_3_0_VERSION
}

PREMIS_VERSIONS_MAP = {
    PREMIS_2_2_VERSION: {
        'namespaces': PREMIS_2_2_NAMESPACES,
        'meta': PREMIS_2_2_META
    },
    PREMIS_3_0_VERSION: {
        'namespaces': PREMIS_3_0_NAMESPACES,
        'meta': PREMIS_3_0_META
    }
}

# Treat PREMIS v. 2.2 as the default (for now).
PREMIS_VERSION = PREMIS_2_2_VERSION
NAMESPACES = PREMIS_VERSIONS_MAP[PREMIS_VERSION]['namespaces']
PREMIS_META = PREMIS_VERSIONS_MAP[PREMIS_VERSION]['meta']
PREMIS_SCHEMA_LOCATION = PREMIS_2_2_SCHEMA_LOCATION


def lxmlns(arg, premis_version=PREMIS_VERSION):
    """Return XPath-usable namespace."""
    namespaces = PREMIS_VERSIONS_MAP[premis_version]['namespaces']
    return '{' + namespaces[arg] + '}'


def snake_to_camel_cap(snake):
    """Convert snake_case to CamelCaseCapitalized."""
    return ''.join([word.capitalize() for word in snake.split('_')])


def snake_to_camel(snake):
    """Convert snake_case to camelCase."""
    tmp = snake_to_camel_cap(snake)
    return tmp[0].lower() + tmp[1:]


def camel_to_snake(camel):
    """Convert camelCase to snake_case."""
    ret = []
    last_lower = False
    for char in camel:
        current_upper = char.upper() == char
        if current_upper and last_lower:
            ret.append('_')
            ret.append(char.lower())
        else:
            ret.append(char.lower())
        last_lower = not current_upper
    return ''.join(ret)



# -*- coding: utf-8 -*-
"""PREMIS reader and writer."""

from __future__ import absolute_import
import logging

from .premis import (
    PREMISElement,
    PREMISObject,
    PREMISEvent,
    PREMISAgent,
    data_to_premis,
    premis_to_data,
    data_find,
    data_find_all,
    data_find_text,
    data_find_text_or_all
)
from .utils import (
    XSI_NAMESPACE,
    PREMIS_2_2_VERSION,
    PREMIS_2_2_NAMESPACE,
    PREMIS_2_2_XSD,
    PREMIS_2_2_SCHEMA_LOCATION,
    PREMIS_2_2_NAMESPACES,
    PREMIS_2_2_META,
    PREMIS_3_0_VERSION,
    PREMIS_3_0_NAMESPACE,
    PREMIS_3_0_XSD,
    PREMIS_3_0_SCHEMA_LOCATION,
    PREMIS_3_0_NAMESPACES,
    PREMIS_3_0_META,
    PREMIS_VERSIONS_MAP,
    PREMIS_VERSION,
    NAMESPACES,
    PREMIS_META,
    PREMIS_SCHEMA_LOCATION,
    lxmlns,
    snake_to_camel_cap,
    snake_to_camel,
    camel_to_snake
)


LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

__all__ = ['PREMISElement', 'PREMISObject', 'PREMISEvent', 'PREMISAgent',
           'data_to_premis', 'premis_to_data', 'data_find', 'data_find_all',
           'data_find_text', 'data_find_text_or_all', 'XSI_NAMESPACE',
           'PREMIS_2_2_VERSION', 'PREMIS_2_2_NAMESPACE', 'PREMIS_2_2_XSD',
           'PREMIS_2_2_SCHEMA_LOCATION', 'PREMIS_2_2_NAMESPACES',
           'PREMIS_2_2_META', 'PREMIS_3_0_VERSION', 'PREMIS_3_0_NAMESPACE',
           'PREMIS_3_0_XSD', 'PREMIS_3_0_SCHEMA_LOCATION',
           'PREMIS_3_0_NAMESPACES', 'PREMIS_3_0_META', 'PREMIS_VERSIONS_MAP',
           'PREMIS_VERSION', 'NAMESPACES', 'PREMIS_META',
           'PREMIS_SCHEMA_LOCATION', 'lxmlns', 'snake_to_camel_cap',
           'snake_to_camel', 'camel_to_snake']



# -*- coding: utf-8 -*-
"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import codecs
from os import path
import re

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))


def read(*parts):
    with codecs.open(path.join(here, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


# Get the long description from the relevant file
with codecs.open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='metsrw',
    version=find_version('metsrw', '__init__.py'),

    description='Library for dealing with METS files.',
    long_description=long_description,

    url='https://github.com/artefactual-labs/mets-reader-writer/',

    author='Artefactual',
    author_email='info@artefactual.com',

    license='AGPL',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],

    keywords='mets',

    packages=find_packages(exclude=['docs', 'fixtures', 'requirements', 'tests*']),

    install_requires=['future', 'lxml', 'six'],

    include_package_data=True,
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*',
)



# -*- coding: utf-8 -*-
"""METS reader and writer."""

from __future__ import absolute_import
import logging

from .exceptions import MetsError, ParseError
from .fsentry import FSEntry
from .metadata import AMDSec, SubSection, MDRef, MDWrap
from .mets import METSDocument
from .utils import (
    NAMESPACES,
    SCHEMA_LOCATIONS,
    lxmlns,
    FILE_ID_PREFIX,
    GROUP_ID_PREFIX,
    urlencode,
    urldecode,
)
from .validate import (
    METS_XSD_PATH,
    AM_SCT_PATH,
    AM_PNTR_SCT_PATH,
    get_schematron,
    validate,
    get_xmlschema,
    xsd_validate,
    schematron_validate,
    sct_report_string,
    xsd_error_log_string,
    report_string
)
from .di import (
    FeatureBroker,
    set_feature_broker_to_default_state,
    feature_broker,
    Dependency,
    has_class_methods,
    has_methods,
    is_class
)
from . import plugins

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())
__version__ = '0.2.4'

__all__ = [
    'AMDSec',
    'AM_PNTR_SCT_PATH',
    'AM_SCT_PATH',
    'Dependency',
    'FILE_ID_PREFIX',
    'FSEntry',
    'FeatureBroker',
    'GROUP_ID_PREFIX',
    'MDRef',
    'MDWrap',
    'METSDocument',
    'METS_XSD_PATH',
    'MetsError',
    'NAMESPACES',
    'ParseError',
    'SCHEMA_LOCATIONS',
    'SubSection',
    '__version__',
    'feature_broker',
    'get_schematron',
    'get_xmlschema',
    'has_class_methods',
    'has_methods',
    'is_class',
    'lxmlns',
    'plugins',
    'report_string',
    'schematron_validate',
    'sct_report_string',
    'set_feature_broker_to_default_state',
    'urldecode',
    'urlencode',
    'validate',
    'xsd_error_log_string',
    'xsd_validate',
]
